@@ -25,6 +25,7 @@ FuzzHelper::FuzzHelper(unsigned char const* data, size_t size) :
 void
 FuzzHelper::doChecks()
 {
+    Pl_RunLength::setMemoryLimit(1'000'000);
     Pl_Discard discard;
     Pl_RunLength p("decode", &discard, Pl_RunLength::a_decode);
     p.write(const_cast<unsigned char*>(data), size);
@@ -37,7 +38,7 @@ FuzzHelper::run()
     try {
         doChecks();
     } catch (std::runtime_error const& e) {
-        std::cerr << "runtime_error: " << e.what() << std::endl;
+        std::cerr << "runtime_error: " << e.what() << '\n';
     }
 }
 
